@@ -1,6 +1,6 @@
 /*
  * This file is part of the Green End SFTP Server.
- * Copyright (C) 2007, 2010, 2011 Richard Kettlewell
+ * Copyright (C) 2007, 2010, 2011, 2014 Richard Kettlewell
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,8 +82,8 @@ static int inputline;
 static int progress_indicators = 1;
 static int terminal_width;
 static int textmode;
-static const char *newline = "\r\n";
-static const char *vendorname, *servername, *serverversion, *serverversions;
+static char *newline;
+static char *vendorname, *servername, *serverversion, *serverversions;
 static uint64_t serverbuild;
 static int stoponerror;
 static int echo;
@@ -365,26 +365,32 @@ static int sftp_init(void) {
     D(("server sent extension '%s'", xname));
     /* TODO table-driven extension parsing */
     if(!strcmp(xname, "newline")) {
+      free(newline);
       newline = xstrdup(xdata);
       if(!*newline)
         return error("cannot cope with empty newline sequence");
       /* TODO check newline sequence doesn't contain repeats */
     } else if(!strcmp(xname, "vendor-id")) {
       struct sftpjob xjob;
+      char *vn ,*sn, *sv;
       
       xjob.a = &allocator;
       xjob.ptr = (void *)xdata;
       xjob.left = xdatalen;
-      cpcheck(sftp_parse_string(&xjob, (char **)&vendorname, 0));
-      cpcheck(sftp_parse_string(&xjob, (char **)&servername, 0));
-      cpcheck(sftp_parse_string(&xjob, (char **)&serverversion, 0));
+      cpcheck(sftp_parse_string(&xjob, &vn, 0));
+      cpcheck(sftp_parse_string(&xjob, &sn, 0));
+      cpcheck(sftp_parse_string(&xjob, &sv, 0));
       cpcheck(sftp_parse_uint64(&xjob, &serverbuild));
-      vendorname = xstrdup(vendorname);
-      servername = xstrdup(servername);
-      serverversion = xstrdup(serverversion);
-    } else if(!strcmp(xname, "versions"))
+      free(vendorname);
+      vendorname = xstrdup(vn);
+      free(servername);
+      servername = xstrdup(sn);
+      free(serverversion);
+      serverversion = xstrdup(sv);
+    } else if(!strcmp(xname, "versions")) {
+      free(serverversions);
       serverversions = xstrdup(xdata);
-    else if(!strcmp(xname, "symlink-order@rjk.greenend.org.uk")) {
+    } else if(!strcmp(xname, "symlink-order@rjk.greenend.org.uk")) {
       /* See commentary in v3.c */
       if(!strcmp(xdata,  "targetpath-linkpath"))
         quirk_reverse_symlink = 1;
@@ -420,11 +426,15 @@ static int sftp_init(void) {
       cpcheck(sftp_parse_uint16(&xjob, &u16)); /* supported-open-block-vector */
       cpcheck(sftp_parse_uint16(&xjob, &u16)); /* supported-block-vector */
       cpcheck(sftp_parse_uint32(&xjob, &u32)); /* attrib-extension-count */
-      while(u32-- > 0)
+      while(u32 > 0) {
         cpcheck(sftp_parse_string(&xjob, 0, 0)); /* attrib-extension-names */
+        --u32;
+      }
       cpcheck(sftp_parse_uint32(&xjob, &u32)); /* extension-count */
-      while(u32-- > 0)
+      while(u32 > 0) {
         cpcheck(sftp_parse_string(&xjob, 0, 0)); /* extension-names */
+        --u32;
+      }
     }
   }
   /* Make sure outbound translation will actually work */
@@ -548,9 +558,11 @@ static int sftp_readdir(const struct client_handle *hp,
     if(n > SIZE_MAX / sizeof(struct sftpattr))
       fatal("too many attributes in SSH_FXP_READDIR response");
     attrs = sftp_alloc(fakejob.a, n * sizeof(struct sftpattr));
-    *nattrsp = n;
-    *attrsp = attrs;
-    while(n-- > 0) {
+    if(nattrsp)
+      *nattrsp = n;
+    if(attrsp)
+      *attrsp = attrs;
+    while(n > 0) {
       cpcheck(sftp_parse_path(&fakejob, &name));
       if(protocol->version <= 3)
         cpcheck(sftp_parse_path(&fakejob, &longname));
@@ -559,13 +571,16 @@ static int sftp_readdir(const struct client_handle *hp,
       attrs->longname = longname;
       attrs->wname = sftp_mbs2wcs(attrs->name);
       ++attrs;
+      --n;
     }
     return 0;
   case SSH_FXP_STATUS:
     cpcheck(sftp_parse_uint32(&fakejob, &n));
     if(n == SSH_FX_EOF) {
-      *nattrsp = 0;
-      *attrsp = 0;
+      if(nattrsp)
+        *nattrsp = 0;
+      if(attrsp)
+        *attrsp = 0;
       return 0;
     }
     status();
@@ -965,19 +980,21 @@ static int sort_by_mtime(const void *av, const void *bv) {
 
 /* Reverse an array in place */
 static void reverse(void *array, size_t count, size_t size) {
-  void *tmp = xmalloc(size);
-  char *const base = array;
-  size_t n;
-  
-  /* If size is even then size/2 is the first half of the array and that's fine.
-   * If size is odd then size/2 goes up to but excludes the middle member
-   * which is also fine. */
-  for(n = 0; n < size / 2; ++n) {
-    memcpy(tmp, base + n * size, size);
-    memcpy(base + n * size, base + (count - n - 1) * size, size);
-    memcpy(base + (count - n - 1) * size, tmp, size);
+  if(array) {
+    void *tmp = xmalloc(size);
+    char *const base = array;
+    size_t n;
+
+    /* If size is even then size/2 is the first half of the array and that's fine.
+     * If size is odd then size/2 goes up to but excludes the middle member
+     * which is also fine. */
+    for(n = 0; n < size / 2; ++n) {
+      memcpy(tmp, base + n * size, size);
+      memcpy(base + n * size, base + (count - n - 1) * size, size);
+      memcpy(base + (count - n - 1) * size, tmp, size);
+    }
+    free(tmp);
   }
-  free(tmp);
 }
 
 static int cmd_ls(int ac,
@@ -1834,8 +1851,6 @@ static int cmd_put(int ac,
   ferrcheck(pthread_mutex_lock(&w.m));
   offset = 0;
   while(!w.failed && !eof && !failed) {
-    int wrote;
-    
     /* Wait until we're allowed to send another request */
     if(w.outstanding >= nrequests) {
       ferrcheck(pthread_cond_wait(&w.c2, &w.m));
@@ -1850,7 +1865,6 @@ static int cmd_put(int ac,
     sftp_send_bytes(&fakeworker, h.data, h.len);
     sftp_send_uint64(&fakeworker, offset);
     sftp_send_need(fakejob.worker, buffersize + 4);
-    wrote = 0;
     if(textmode) {
       char *const start = ((char *)fakejob.worker->buffer
                            + fakejob.worker->bufused + 4);
@@ -2344,18 +2358,18 @@ static int cmd_overlap(int attribute((unused)) ac,
   static const char d[] = "dddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd";
   static const char expect[] = "aaaaaaaaaaaaaaaabbbbbbbbbbbbbbbbccccccccccccccccdddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd";
   char buffer[128];
-  int n, r;
+  int n, r, rc = -1;
   uint32_t ida, idb, idc, idd;
-  int fd;
+  int fd = -1;
   struct client_handle h;
 
   memset(&attrs, 0, sizeof attrs);
   if(sftp_open("dest", ACE4_WRITE_DATA,
                SSH_FXF_CREATE_TRUNCATE, &attrs, &h))
-    return -1;
+    goto error;
   if((fd = open("dest", O_RDWR)) < 0) {
     perror("open dest");
-    return -1;
+    goto error;
   }
   for(n = 0; n < 1024; ++n) {
     sftp_send_begin(&fakeworker);
@@ -2398,27 +2412,31 @@ static int cmd_overlap(int attribute((unused)) ac,
 
     if(lseek(fd, 0, SEEK_SET) < 0) {
       perror("lseek");
-      return -1;
+      goto error;
     }
-    r = read(fd, buffer, 128);
+    r = read(fd, buffer, sizeof buffer);
     if(r != 48+64) {
       fprintf(stderr, "expected %d bytes got %d\n", 48+64, r);
-      return -1;
+      goto error;
     }
+    buffer[r] = 0;                      /* ensure buffer terminated */
     if(memcmp(buffer, expect, 48+64)) {
       fprintf(stderr, "buffer contents mismatch\n"
               "expect: %s\n"
-              "   got: %.112s\n",
+              "   got: %s\n",
               expect, buffer);
-      return -1;
+      goto error;
     }
     if(ftruncate(fd, 0) < 0) {
       perror("ftruncate");
-      return -1;
+      goto error;
     }
   }
-  close(fd);
-  return 0;
+  rc = 0;
+error:
+  if(fd >= 0)
+    close(fd);
+  return rc;
 }
 
 
@@ -2782,6 +2800,7 @@ int main(int argc, char **argv) {
 #endif
 
   setlocale(LC_ALL, "");
+  newline = xstrdup("\r\n");
 
   /* Figure out terminal width */
   {
