@@ -1,6 +1,6 @@
 /*
  * This file is part of the Green End SFTP Server.
- * Copyright (C) 2007, 2009, 2011 Richard Kettlewell
+ * Copyright (C) 2007, 2009, 2011, 2014 Richard Kettlewell
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,6 +50,9 @@
 #include <stdio.h>
 #include <syslog.h>
 #include <netinet/in.h>
+#if HAVE_SYS_PRCTL_H
+# include <sys/prctl.h>
+#endif
 
 /* Linux and BSD have daemon() but other UNIX platforms tend not to */
 #if ! HAVE_DAEMON
@@ -495,6 +498,11 @@ int main(int argc, char **argv) {
    */
   signal(SIGPIPE, SIG_IGN);
 
+#if HAVE_PRCTL
+  if(prctl(PR_SET_DUMPABLE, 0L) < 0)
+    fatal("error calling prctl: %s", strerror(errno));
+#endif
+
 #if DAEMON
   if(user) {
     /* Look up the user */
